#include <avr/io.h>
#include <string.h>
#include <util/delay_basic.h>
#define F_CPU 8000000UL  
#include <util/delay.h>
#include <util/twi.h>
#include <avr/interrupt.h> 
#include <avr/wdt.h> 
#include <avr/eeprom.h>




 //                         76543210
#define K1V_1 PORTC |=0b00010000
#define K2V_1 PORTD |=0b00000010
#define K3V_1 PORTD |=0b00000100
#define K1I_1 PORTD |=0b01000000
#define K2I_1 PORTD |=0b00100000
#define K3I_1 PORTD |=0b00010000
#define K1V_0 PORTC &=0b11101111
#define K2V_0 PORTD &=0b11111101
#define K3V_0 PORTD &=0b11111011
#define K1I_0 PORTD &=0b10111111
#define K2I_0 PORTD &=0b11011111
#define K3I_0 PORTD &=0b11101111

// 76543210
#define seg_A_1 PORTC |=0b00100000
#define seg_B_1 PORTD |=0b00001000
#define seg_C_1 PORTB |=0b00000001
#define seg_D_1 PORTB |=0b00000100
#define seg_E_1 PORTC |=0b00000100
#define seg_F_1 PORTD |=0b00000001
#define seg_G_1 PORTD |=0b10000000
#define seg_DP_1 PORTB |=0b00000010
#define seg_A_0 PORTC &=0b11011111
#define seg_B_0 PORTD &=0b11110111
#define seg_C_0 PORTB &=0b11111110
#define seg_D_0 PORTB &=0b11111011
#define seg_E_0 PORTC &=0b11111011
#define seg_F_0 PORTD &=0b11111110
#define seg_G_0 PORTD &=0b01111111
#define seg_DP_0 PORTB &=0b11111101

// 76543210
#define led_A_0 PORTB |=0b01000000
#define led_mA_0 PORTB |=0b10000000
#define led_A_1 PORTB &=0b10111111
#define led_mA_1 PORTB &=0b01111111



#define ADC_VREF_TYPE 0xC0

#define Vref 2560   //mv
#define precount 240





unsigned char segment[50];
unsigned char digit1,digit2,digit3;
unsigned char segment_online;
unsigned int iter;
unsigned char flag_volt;
unsigned char flag_amper;
unsigned char coef_u;
unsigned char coef_i;
unsigned int volt,amper;

unsigned char digit[10]={0b11111100,0b01100000,0b11011010,0b11110010,0b01100110,0b10110110,0b10111110,0b11100000,
						0b11111110,0b11110110};//
unsigned int iteration[400];


// Timer 0 overflow interrupt service routine
ISR (TIMER0_OVF_vect)
{

TCNT0=precount;
unsigned char dg;
dg=segment_online/8;
seg_A_1;seg_B_1;seg_C_1;seg_D_1;seg_E_1;seg_F_1;seg_G_1;seg_DP_1;
K1V_0;K2V_0;K3V_0;K1I_0;K2I_0;K3I_0;
switch (dg)
	{
	case 0:
	K1V_1;
	break;
	
	case 1:
	K2V_1;
	break;
	
	case 2:
	K3V_1;
	break;
	
	case 3:
	K1I_1;
	break;
	
	case 4:
	K2I_1;
	break;
	
	case 5:
	K3I_1;
	break;
	}

switch (segment_online)
	{
	case 0:case 8:case 16:case 24:case 32:case 40:
	if(segment[segment_online]==1)seg_A_0;
	break;
	
	case 1:case 9:case 17:case 25:case 33:case 41:
	if(segment[segment_online]==1)seg_B_0;
	break;
	
	case 2:case 10:case 18:case 26:case 34:case 42:
	if(segment[segment_online]==1)seg_C_0;
	break;
	
	case 3:case 11:case 19:case 27:case 35:case 43:
	if(segment[segment_online]==1)seg_D_0;
	break;
	
	case 4:case 12:case 20:case 28:case 36:case 44:
	if(segment[segment_online]==1)seg_E_0;
	break;
	
	case 5:case 13:case 21:case 29:case 37:case 45:
	if(segment[segment_online]==1)seg_F_0;
	break;
	
	case 6:case 14:case 22:case 30:case 38:case 46:
	if(segment[segment_online]==1)seg_G_0;
	break;
	
	case 7:case 15:case 23:case 31:case 39:case 47:
	if(segment[segment_online]==1)seg_DP_0;
	break;
	}

segment_online++;
if(segment_online>47){segment_online=0;}

}

//





// ADC interrupt service routine
// with auto input scanning
ISR (ADC_vect)
{
if(iter<400)iteration[iter]=ADCW;
if(iter==0){ADMUX=0xC1;_delay_us(30);flag_volt=0;}//1 V
if(iter==100){ADMUX=0xCF;_delay_us(30);flag_volt=1;}//GND
if(iter==300){ADMUX=0xC0;_delay_us(30);flag_amper=0;}//3 A
if(iter==400){ADMUX=0xCF;_delay_us(30);flag_amper=1;}//GND

iter++;
if(iter>599)iter=0;
ADCSRA|=0x40;


}

// n-    

void decimal_digit (unsigned int dig) //
	{
	if(dig>999)dig=dig/1000;
	unsigned int b,c,d;
	b=dig;					
	
	c=b/100;
	digit1=digit[c];
	d=c*100;
	b=b-d;
	c=b/10;
	digit2=digit[c];
	d=c*10;
	b=b-d;
	digit3=digit[b];
	}
	

	
	
//*********************************************************


//


int main(void)
{
unsigned char cnt,char1,char2,char3,a,b;
unsigned long int adc_summ;


// Declare your local variables here

// Input/Output Ports initialization
// Port B initialization
// Func7=Out Func6=Out Func5=In Func4=In Func3=In Func2=Out Func1=Out Func0=Out 
// State7=0 State6=0 State5=T State4=T State3=T State2=0 State1=0 State0=0 
PORTB=0xFF;
DDRB=0xC7;

// Port C initialization
// Func6=In Func5=Out Func4=Out Func3=In Func2=Out Func1=In Func0=In 
// State6=P State5=0 State4=1 State3=P State2=0 State1=T State0=T 
PORTC=0x6C;
DDRC=0x34;

// Port D initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
// State7=0 State6=1 State5=1 State4=1 State3=0 State2=1 State1=1 State0=0 
PORTD=0x09;
DDRD=0xFF;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 125,000 kHz
TCCR0=0x03;
TCNT0=precount;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: Int., cap. on AREF
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0xCE;    



// Global enable interrupts
sei();

while (1)
	{
	
	
		if(flag_volt==1)
		{
		adc_summ=0;	
		for(cnt=0;cnt<100;cnt++)
			{
			adc_summ=adc_summ+iteration[cnt];
			}
		adc_summ=adc_summ/100;
		volt=(adc_summ*Vref/1024);
		
		
		if(volt<1000)
			{
			decimal_digit(volt);
			segment[7]=1;segment[15]=0;
			}
		else
			{
			decimal_digit(volt/10);
			segment[15]=1;segment[7]=0;
			}
	
		char1=digit1;
		char2=digit2;
		char3=digit3;
	
		for(cnt=0;cnt<3;cnt++)
			{
			switch (cnt)
				{
				case 0:
				b=0b10000000;
				for(a=0;a<7;a++)
					{
					char1=char1 & b;
					if(char1!=0)segment[a]=1;
					else segment[a]=0;
					char1=digit1;
					b=b>>1;
					}
				break;
				
				case 1:
				b=0b10000000;
				for(a=0;a<7;a++)
					{
					char2=char2 & b;
					if(char2!=0)segment[a+8]=1;
					else segment[a+8]=0;
					char2=digit2;
					b=b>>1;
					}
				break;
				
				case 2:
				b=0b10000000;
				for(a=0;a<7;a++)
					{
					char3=char3 & b;
					if(char3!=0)segment[a+16]=1;
					else segment[a+16]=0;
					char3=digit3;
					b=b>>1;
					}
				break;
				
				}
			}
		}
	
		if(flag_amper==1)
		{
		adc_summ=0;	
		for(cnt=0;cnt<100;cnt++)
			{
			adc_summ=adc_summ+iteration[cnt+300];
			}
		adc_summ=adc_summ/100;
		amper=(adc_summ*Vref/1024);
		
		
		if(amper<1000)
			{
			decimal_digit(amper);
			segment[31]=0;led_mA_1;led_A_0;
			}
		else
			{
			decimal_digit(amper/10);
			segment[31]=1;led_mA_0;led_A_1;
			}
		char1=digit1;
		char2=digit2;
		char3=digit3;
		
		for(cnt=0;cnt<3;cnt++)
			{
			switch (cnt)
				{
				case 0:
				b=0b10000000;
				for(a=0;a<7;a++)
					{
					char1=char1 & b;
					if(char1!=0)segment[a+24]=1;
					else segment[a+24]=0;
					char1=digit1;
					b=b>>1;
					}
				break;
				
				case 1:
				b=0b10000000;
				for(a=0;a<7;a++)
					{
					char2=char2 & b;
					if(char2!=0)segment[a+32]=1;
					else segment[a+32]=0;
					char2=digit2;
					b=b>>1;
					}
				break;
				
				case 2:
				b=0b10000000;
				for(a=0;a<7;a++)
					{
					char3=char3 & b;
					if(char3!=0)segment[a+40]=1;
					else segment[a+40]=0;
					char3=digit3;
					b=b>>1;
					}
				break;
				
				}
			}
		}
	}
}
